#include "kernel.h"
#include <stdio.h>

/* The follwing copied from
 * base_unix_sample_main.c.with_gl by nmd 1998-03-5 and
 * modified so that it prints the data in the following format:
 *
 *  Instead of the usual: format is what you might expect:
 *
 *        a b c  d e f  ... = 2 pi i
 * 
 *  means
 *        a*log(z0) + b*log(1/(1-z0)) + c*((z0-1)/z) + d*log(z1) +... = 2 pi i
 *
 *  the format is 
 *     a b   c d   e f = (-1)^n
 *
 * means:
 *  
 *  z0^a (1 - z0)^b z1^c (1 -z1)^b ... = (-1)^n
 *
 */



void nathans_print_complex_equations(
    Triangulation   *manifold)
{
    int             *eqn,

                    i,
      coef[2],
       a, b;
    EdgeClass       *edge;
    PositionedTet   ptet0,
                    ptet;
    Cusp            *cusp;
    Tetrahedron     *tet;
    VertexIndex     v;
    FaceIndex       f,
                    ff;
    PeripheralCurve c;
    int n;

    eqn = NEW_ARRAY(3 * manifold->num_tetrahedra, int);

    /*
     *  Print the edge equations to stdout.
     */

    printf("edge equations:\n");
    for (   edge = manifold->edge_list_begin.next;
            edge != &manifold->edge_list_end;
            edge = edge->next)
    {
        for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
            eqn[i] = 0;

        set_left_edge(edge, &ptet0);
        ptet = ptet0;
        do
        {
            eqn[3*ptet.tet->index +
edge3_between_faces[ptet.near_face][ptet.left_face]]++;
            veer_left(&ptet);
        } while (same_positioned_tet(&ptet, &ptet0) == FALSE);

	n = 0;
	
        for (i = 0; i <  manifold->num_tetrahedra; i++)
        {  
	  a =  eqn[3*i] - eqn[3*i + 2];
          b = -eqn[3*i + 1] + eqn[3*i +2];
	  if (a >= 0)
	    printf(" %d,", a);
	  else
	    printf("%d,", a);
	  if (b >= 0)
	    printf("  %d, ", b);
	  else
	    printf(" %d, ", b);
	  n =n + eqn[3*i +2];
        }
        printf(" = (-1)^(%d) \n", n%2);
    }

    /*
     *  Print the cusp equations to stdout.
     */

    printf("cusp equations:\n");

    for (cusp = manifold->cusp_list_begin.next;
         cusp != &manifold->cusp_list_end;
         cusp = cusp->next)
    {
        if (Dehn_coefficients_are_integers(cusp) == FALSE)
        {
            printf("coefficients not integers on cusp\n");
            continue;
        }

        if (cusp->is_complete)
        {
            coef[M] = 1;
            coef[L] = 0;
        }
        else
        {
            coef[M] = cusp->m;
            coef[L] = cusp->l;
        }

        for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
            eqn[i] = 0;

        for (tet = manifold->tet_list_begin.next;
             tet != &manifold->tet_list_end;
             tet = tet->next)
            for (v = 0; v < 4; v++)
            {
                if (tet->cusp[v] != cusp)
                    continue;

                for (f = 0; f < 4; f++)
                {
                    if (f == v)
                        continue;

                    ff = remaining_face[v][f];

                    for (c = 0; c < 2; c++) /* c = M, L */
                        eqn[3*tet->index + edge3_between_faces[f][ff]]
                            += coef[c] *
FLOW(tet->curve[c][right_handed][v][f],
tet->curve[c][right_handed][v][ff]);
                }
            }

		if( cusp->is_complete )
        {
            coef[M] = 0;
            coef[L] = 1;
	       
	    n = 0;
	
	    for (i = 0; i <  manifold->num_tetrahedra; i++)
	      {  
		a =  eqn[3*i] - eqn[3*i + 2];
		b = -eqn[3*i + 1] + eqn[3*i +2];
		if (a >= 0)
		  printf(" %d,", a);
		else
		  printf("%d,", a);
		if (b >= 0)
		  printf("  %d, ", b);
		else
		  printf(" %d, ", b);
		n =n + eqn[3*i +2];
	      }
	    printf(" = (-1)^(%d) h(m) \n", n%2);
	    	        
	         for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
	            eqn[i] = 0;
	
	        for (tet = manifold->tet_list_begin.next;
	             tet != &manifold->tet_list_end;
	             tet = tet->next)
	            for (v = 0; v < 4; v++)
	            {
	                if (tet->cusp[v] != cusp)
	                    continue;
	
	                for (f = 0; f < 4; f++)
	                {
	                    if (f == v)
	                        continue;
	
	                    ff = remaining_face[v][f];
	
	                    for (c = 0; c < 2; c++) /* c = M, L */
	                        eqn[3*tet->index + edge3_between_faces[f][ff]]
	                            += coef[c] *
	FLOW(tet->curve[c][right_handed][v][f],
	tet->curve[c][right_handed][v][ff]);
	                }
	            }
	
		n = 0;
	
		for (i = 0; i <  manifold->num_tetrahedra; i++)
		  {  
		     a =  eqn[3*i] - eqn[3*i + 2];
		     b = -eqn[3*i + 1] + eqn[3*i +2];
		     if (a >= 0)
		       printf(" %d,", a);
		     else
		       printf("%d,", a);
		     if (b >= 0)
		       printf("  %d, ", b);
		     else
		       printf(" %d, ", b);
		     n =n + eqn[3*i +2];
		  }
		printf(" = (-1)^(%d) h(l)  \n", n%2);


		}
    }

    my_free(eqn);
}

/* This time it this form
 *
 *        a b c  d e f  ... = 2 pi i
 *
 *  means
 *       
 *     a*log(z0) + b*log(1/(1-z0)) + c*log((z0-1)/z) + d*log(z1) +... = 2 pi i
 *
 *  for the cusp equations, have h(m), h(l) of cusp 0, then of cusp 1 etc.
 */

int  std_complex_equations_to_file(
    Triangulation   *manifold, char* file_name )
{
    int             *eqn,

                    i,
      coef[2],
       a, b;
    EdgeClass       *edge;
    PositionedTet   ptet0,
                    ptet;
    Cusp            *cusp;
    Tetrahedron     *tet;
    VertexIndex     v;
    FaceIndex       f,
                    ff;
    PeripheralCurve c;
    int n;
    FILE* fp;
    fp = fopen(file_name, "w");
    if (fp == NULL)
      return 0;

    eqn = NEW_ARRAY(3 * manifold->num_tetrahedra, int);

    /*
     *  Build edge equations.
     */

    fprintf(fp, "edge equations:\n");
    for (   edge = manifold->edge_list_begin.next;
            edge != &manifold->edge_list_end;
            edge = edge->next)
      {
        for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
	  eqn[i] = 0;

        set_left_edge(edge, &ptet0);
        ptet = ptet0;
        do{
	  eqn[3*ptet.tet->index + edge3_between_faces[ptet.near_face][ptet.left_face]]++;
	  veer_left(&ptet);
        } while (same_positioned_tet(&ptet, &ptet0) == FALSE);

        for (i = 0; i <  manifold->num_tetrahedra; i++)
        {  
	  a =  eqn[3*i];
	  b = eqn[3*i + 1];
	  c = eqn[3*i + 2];
	  fprintf(fp, "%d %d %d ", a, b, c);
        }
        fprintf(fp, "\n");
      }

    /*
     *  Print the cusp equations to stdout.
     */

    fprintf(fp, "cusp equations:\n");

    for (cusp = manifold->cusp_list_begin.next;
         cusp != &manifold->cusp_list_end;
         cusp = cusp->next)
    {
        if (Dehn_coefficients_are_integers(cusp) == FALSE)
        {
            fprintf(fp, "coefficients not integers on cusp\n");
            continue;
        }

        if (cusp->is_complete)
        {
            coef[M] = 1;
            coef[L] = 0;
        }
        else
        {
            coef[M] = cusp->m;
            coef[L] = cusp->l;
        }

        for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
            eqn[i] = 0;

        for (tet = manifold->tet_list_begin.next;
             tet != &manifold->tet_list_end;
             tet = tet->next)
            for (v = 0; v < 4; v++)
            {
                if (tet->cusp[v] != cusp)
                    continue;

                for (f = 0; f < 4; f++)
                {
                    if (f == v)
                        continue;

                    ff = remaining_face[v][f];

                    for (c = 0; c < 2; c++) /* c = M, L */
                        eqn[3*tet->index + edge3_between_faces[f][ff]]
                            += coef[c] *
FLOW(tet->curve[c][right_handed][v][f],
tet->curve[c][right_handed][v][ff]);
                }
            }

		if( cusp->is_complete )
        {
            coef[M] = 0;
            coef[L] = 1;
	       
	    n = 0;
	
	    for (i = 0; i <  manifold->num_tetrahedra; i++)
	      {  
		a =  eqn[3*i];
		b = eqn[3*i + 1];
		c = eqn[3*i + 2];
		fprintf(fp, "%d %d %d ", a, b, c);
	      }
	    fprintf(fp, " \n");
	    	        
	         for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
	            eqn[i] = 0;
	
	        for (tet = manifold->tet_list_begin.next;
	             tet != &manifold->tet_list_end;
	             tet = tet->next)
	            for (v = 0; v < 4; v++)
	            {
	                if (tet->cusp[v] != cusp)
	                    continue;
	
	                for (f = 0; f < 4; f++)
	                {
	                    if (f == v)
	                        continue;
	
	                    ff = remaining_face[v][f];
	
	                    for (c = 0; c < 2; c++) /* c = M, L */
	                        eqn[3*tet->index + edge3_between_faces[f][ff]]
	                            += coef[c] *
	FLOW(tet->curve[c][right_handed][v][f],
	tet->curve[c][right_handed][v][ff]);
	                }
	            }
	
		n = 0;
	
		for (i = 0; i <  manifold->num_tetrahedra; i++)
		  {  
		    a =  eqn[3*i];
		    b = eqn[3*i + 1];
		    c = eqn[3*i + 2];
		    fprintf(fp, "%d %d %d ", a, b, c);
		  }
		fprintf(fp, "\n");


		}
    }
    fclose(fp);
    my_free(eqn);
    return 1;
}

/* The follwing copied from
 * base_unix_sample_main.c.with_gl by nmd 1998-03-05 and
 * modified so creates a script for Mathematica to compute
 * the normal boundary slopes.  See previous function for the 
 * output format.
 */

void create_normal_surface_script(
    Triangulation   *manifold)
{
    int             *eqn,
                    i,
                    coef[2];
    EdgeClass       *edge;
    PositionedTet   ptet0,
                    ptet;
    Cusp            *cusp;
    Tetrahedron     *tet;
    VertexIndex     v;
    FaceIndex       f,
                    ff;
    PeripheralCurve c;
    int n;

    eqn = NEW_ARRAY(3 * manifold->num_tetrahedra, int);

    /*
     *  Print the edge equations to stdout.
     */

    printf("<< ~nathand/Mathematica/NormalBoundarySlopes.m\n");
    printf("n := %i;\n", manifold->num_tetrahedra);

    printf("r := {\n");
    for (   edge = manifold->edge_list_begin.next;
            edge != &manifold->edge_list_end;
            edge = edge->next)
    {
        for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
            eqn[i] = 0;

        set_left_edge(edge, &ptet0);
        ptet = ptet0;
        do
        {
            eqn[3*ptet.tet->index +
	       edge3_between_faces[ptet.near_face][ptet.left_face]]++;
            veer_left(&ptet);
        } while (same_positioned_tet(&ptet, &ptet0) == FALSE);

        n = 0;
	
        printf("\t{ ");
        for (i = 0; i <  manifold->num_tetrahedra; i++)
        {  
            printf("%d, %d ", eqn[3*i] - eqn[3*i + 2], -eqn[3*i + 1] + eqn[3*i +2]);
            n =n + eqn[3*i +2];
            if(i <  manifold->num_tetrahedra - 1) printf(", ");
        }
        if(edge->next == &manifold->edge_list_end)
	  printf(" }\n");
        else
	  printf(" },\n");
    }
    printf("};\n");

    /*
     *  Print the cusp equations to stdout.
     */

    for (cusp = manifold->cusp_list_begin.next;
         cusp != &manifold->cusp_list_end;
         cusp = cusp->next)
    {
        if (Dehn_coefficients_are_integers(cusp) == FALSE)
        {
            printf("coefficients not integers on cusp\n");
            continue;
        }

        if (cusp->is_complete)
        {
            coef[M] = 1;
            coef[L] = 0;
        }
        else
        {
            coef[M] = cusp->m;
            coef[L] = cusp->l;
        }

        for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
            eqn[i] = 0;

        for (tet = manifold->tet_list_begin.next;
             tet != &manifold->tet_list_end;
             tet = tet->next)
            for (v = 0; v < 4; v++)
            {
                if (tet->cusp[v] != cusp)
                    continue;

                for (f = 0; f < 4; f++)
                {
                    if (f == v)
                        continue;

                    ff = remaining_face[v][f];

                    for (c = 0; c < 2; c++) /* c = M, L */
                        eqn[3*tet->index + edge3_between_faces[f][ff]]
                            += coef[c] *
FLOW(tet->curve[c][right_handed][v][f],
tet->curve[c][right_handed][v][ff]);
                }
            }

		if( cusp->is_complete )
        {
            coef[M] = 0;
            coef[L] = 1;
	       
	    n = 0;
	    
	    printf("m := {");
	    for (i = 0; i <  manifold->num_tetrahedra; i++)
	      {  
		printf("%d, %d", eqn[3*i] - eqn[3*i + 2], -eqn[3*i + 1] + eqn[3*i +2]);
		n =n + eqn[3*i +2];
		if(i < manifold->num_tetrahedra - 1) printf(", ");
	      }
	    printf("};\n");
	    	        
	         for (i = 0; i < 3 * manifold->num_tetrahedra; i++)
	            eqn[i] = 0;
	
	        for (tet = manifold->tet_list_begin.next;
	             tet != &manifold->tet_list_end;
	             tet = tet->next)
	            for (v = 0; v < 4; v++)
	            {
	                if (tet->cusp[v] != cusp)
	                    continue;
	
	                for (f = 0; f < 4; f++)
	                {
	                    if (f == v)
	                        continue;
	
	                    ff = remaining_face[v][f];
	
	                    for (c = 0; c < 2; c++) /* c = M, L */
	                        eqn[3*tet->index + edge3_between_faces[f][ff]]
	                            += coef[c] *
	FLOW(tet->curve[c][right_handed][v][f],
	tet->curve[c][right_handed][v][ff]);
	                }
	            }
	
		n = 0;
		
		printf("l := {");
		for (i = 0; i <  manifold->num_tetrahedra; i++)
		  {  
		    printf("%d, %d", eqn[3*i] - eqn[3*i + 2], -eqn[3*i + 1] + eqn[3*i +2]);
		    n =n + eqn[3*i +2];
		    if(i < manifold->num_tetrahedra - 1) printf(", ");
		  }
		printf("};\n");


		}
    }
    printf("NormalSlopes[n, r, m, l] >> \"%s_ns\";\n", manifold->name);
    my_free(eqn);
}
