/*
 *	unix_UI.c
 *
 *	This file contains a quick and dirty implementation of some UI functions
 *	required for using the SnapPea kernel in a stdio.h environment.
 *	It's intended for use by mathematicians who want to call the SnapPea
 *	kernel functions from within their own C code.
 */

#include "SnapPea.h"
#include <stdio.h>
#include <stdlib.h>

void uAcknowledge(
	char *message)
{
	printf("%s\n", message);
}

void uFatalError(
	char	*function,
	char	*file)
{
	printf("A fatal error has occurred in the function %s() in the file %s.c.\n",
		function, file);
	exit(0);
}

void uAbortMemoryFull(void)
{
	printf("out of memory\n");
	exit(0);
}

int uQuery(
	char	*message,
	int		num_responses,
	char	*responses[],
	int		default_response)
{
	/*
	 *	If desired you could write this function to obtain a response from the user,
	 *	but for now it set up to return the default response, to facilitate batch
	 *	computations.
	 */
	printf("Q: %s\nA:  %s\n", message, responses[default_response]);

	return default_response;
}

/*
 *	The "long computation" feature is unused, but we define its
 *	global variables and functions to avoid a link error.
 */
Boolean	gLongComputationInProgress,
		gLongComputationCancelled;
void uLongComputationBegins(
	char	*message,
	Boolean	is_abortable)
{
}
FuncResult uLongComputationContinues()
{
	return func_OK;
}
void uLongComputationEnds()
{
}
