#
# rep.py
#

from SnapPea import *

# SnapPea computes a presentation of pi_1(M) and matrices in PSL_2(C)
# corresponding to each generator.  It returns these matrices as
# elements of SL_2(C), but these need not give a representation of
# pi_1(M).  This function checks to see if they do.

def is_lift_representation_into_SL_2(manifold):
    pi_1 = fundamental_group(manifold)
    for rel in pi_1.relations():
        M = pi_1.word_to_moebius(rel)
        if abs(M.trace() + 2) < 0.1:
            return 0

    return 1

# The next function tries changing the triangulation so that we do
# have a representation into SL_2(C).  Reports 1 if it succeeds.

def make_lift_representation(manifold):
    for i in range(100):
        if is_lift_representation_into_SL_2(manifold):
            return 1
        randomize_triangulation(manifold)
    return 0

# Test whether the following is true:
#
# Let M be the exterior of a hyperbolic knot in S^3.  Then the trace
# of the longitude of the is -2 for any lift of the complete faithful
# representation into SL(2,C).

def main():
    for manifold in link_complements(1):
        if manifold.complete_solution_type() != geometric_solution:
            continue
        if not make_lift_representation(manifold):
            print "make_lift Failed for %s" % manifold
        pi_1 = fundamental_group(manifold)
        long = pi_1.longitude(0)
        trace = pi_1.word_to_moebius(long).trace()
        if abs(trace - 2) < 0.1:
            print manifold, "HAS TRACE +2!"
        else:
            print manifold, "has trace -2 (as expected)"

#---end of main()----
            
main()
        
#-----program output --------------
#
#  4_1 has trace -2 (as expected)
#  5_2 has trace -2 (as expected)
#  6_1 has trace -2 (as expected)
#  6_2 has trace -2 (as expected)
#
# etc.


